<?php

/* RECORD.PHP - manipulate 'records' and write/read them to/from database

This module "processes" posts as a step between the display code and the 
database: DISPLAY > RECORD > MYSQL. For example:

displayrecord() calls readrecord() which calls dbreadrecord()

dbreadrecord() returns a $record and readrecord() validates and converts the 
record data suitible for displayrecord() to display it via displayhtml()

The following functions are defined:

	readrecord()			// called by DISPLAY and Admin
	writerecord()			// called by Admin
	createcomment()			// creates comment $record from POST
	stringtorecord()		// converts raw post to $record
					//  dbreadrecord() calls this
	recordtostring()		// converts $record to string
	sortrecords()			// sorts list of record ids
	readfilerecord()		// read a "page"
*/


/* readrecord - read an entry (post) by ID and process it into a "record" */

// $data is a way for admin to preview (only ADMIN and DISPLAY call this
// function)

function readrecord($id, $data = NULL, $headers = FALSE) {

	debug("'$id'");

	// first, read file into a "record"

	if ($data)
		$record = stringtorecord($data);
	else
		$record = dbreadrecord($id);

	// $record will be FALSE if record not found - the code actually
	// checks for existence with if (!dblistrecords($id)) so it should
	// never be FALSE

	if ($record == FALSE) {
		$record = createrecord("Error #49",htmlgeterror('NOTPOST'));
		$record['nocomments'] = 1;
	}

	$record['id'] = $id;

// create date and time strings if needed; they should/may exist in a 
// written record but this function is used to preview from data by Admin

	if (!isset($record['date'])) {
		$record['time'] = time();
		$record['date'] = date(config('datestr'),$record['time']);
	}
# this can be annoying; need to document why it's done...
	if (!isset($record['time']))
		$record['time'] = strtotime($record['date']);
# need to ^reset^ time if date changed (is different)...

	$record['datetime'] = date(DATE_RFC3339,$record['time']);

	if (isset($record['title']))
		$record['title'] = htmlspecialchars($record['title'],ENT_QUOTES);
	else
		$record['title'] = '';          // have to have a title

	if (config('headers'))			// if defaults
		_defaultheaders($record);	//  default defaults

	if ($headers) {
		unset($record['body']);
		return $record;
	}

	$no = (isset($record['notranslate'])) ? $record['notranslate'] : 0;
	if ($no === 1)
		return $record;
	$record['body'] = gmlp_convert($record['body'],$no === '');

// a little used, odd thing:
	if ($no === 0)
		_recordrefs($record,$record['body']);

	return $record;
}
/*
NOTES
ALL DEFAULT headers are going to be defined by data:
	[title]
	default = Title
	filter = htmlspecialchars
etc.
*/


/* readfilerecord - read a file in the format of an internal record */

// the pages API is in the PAGES module; pages only have title and date

function readfilerecord($id, $headers = FALSE) {

	$record = dbreadpage($id);
	if ($record == FALSE)
		return createrecord("Error #21 ($id)",htmlgeterror('NOTFOUND'));

	if (!isset($record['date']))
		$record['date'] = date(config('datestr'));

	$record['title'] = htmlspecialchars($record['title'],ENT_QUOTES);

	if ($headers) {
		unset($record['body']);
		return $record;
	}

	$no = (isset($record['notranslate'])) ? $record['notranslate'] : 0;
	if ($no === 1)
		return $record;
	$record['body'] = gmlp_convert($record['body'],$no === '');

	return $record;
}


/* readcomment - used by Admin */

function readcomment($id, $data) {

	debug("'$id'");

	if ($data)
		$record = stringtorecord($data);
	else
		$record = dbreadcomment($id);

	$record['id'] = $id;

	if (!isset($record['date']))
		$record['date'] = date(config('datestr'));

	if (!isset($record['title']))
		$record['title'] = '';

	return $record;
}


/* writerecord - write a record */

// assumes properly formated data from admin form but it adds the basics 
// if they are missing; if $id assumes already validated that it exists

function writerecord($record, $headers = array(), $id = 0) {

	if (!is_array($record))
		$record = stringtorecord($record);

	if (!isset($record['title']))
		$record['title'] = 'Title';

	if (!isset($record['date']))
		$record['date'] = date(config('datestr'));

	$record['time'] = strtotime($record['date']);

	if ($headers)
		foreach ($headers as $k => $v) {
			if (is_numeric($k))
				$record[$v] = '';
			else
				$record[$k] = $v;
		}

	if ($id)
		return dbputrecord($id,$record);

	return dbnewrecord($record);
}


/* stringtorecord - convert record string to associative array */

// assumes properly formated data; it sort of handles missing headers

function stringtorecord($data) {

	$data = explode("\n",$data);

	// PREG_SPLIT_DELIM_CAPTURE is wanted here, but this:
	// $data = preg_split('/(\n)/',$data,-1,PREG_SPLIT_DELIM_CAPTURE);
	// puts the "\n" as a separate array member which is not right

	foreach ($data as &$d)
		$d .= "\n";
	reset($data);
	unset($d);

	while (list( ,$_) = each($data)) {
		if ($_ == "\n") {
			$_ = '';
			break;
		}

		$h = explode(':',$_);
		if (count($h) == 1)
			break;

		$name = array_shift($h);		//  tag: text
		$name = strtolower($name);		//  tag:
		$value = implode(':',$h);		//  tag: Title:Foobar
		$record[$name] = trim($value);
	}

	$record['body'] = $_;
	while (list( ,$_) = each($data))
		$record['body'] .= $_;

	return $record;
}


/* recordtostring - convert record array into a string */

// the string is "key:value\n\nbody" where there can be zero or more of 
// (basically unknown) key:value\n pairs

function recordtostring($record) {

	$data =  '';
	foreach ($record as $key => $value)
		if ($key != 'body')
			$data .= $key.':'.$value."\n";

	return $data."\n".$record['body'];
}


/* sortentries - sort the entries (natural sort is the default) */

function _cmptitle($a, $b) { return strcmp($a['title'],$b['title']); }
function _cmptime($a, $b)  { return strcmp($a['time'],$b['time']);   }

function sortrecords(&$ids, $sort = NULL) {

	if ($sort === NULL)
		$sort = config('sortfunc');

	if ($sort == "date") {
		$i = 0;
		foreach ($ids as $id) {
			$headers[$i] = readrecord($id,NULL,TRUE);
			$headers[$i]['id'] = $id;
			$i++;
		}
		usort($headers,'_cmptime');
		$i = 0;
		foreach ($headers as $h) {
			$ids[$i] = $h['id'];
			$i++;
		}
	}
	else
	if ($sort == 'name') {
		$i = 0;
		foreach ($ids as $id) {
			$headers[$i] = readrecord($id,NULL,TRUE);
			$headers[$i]['id'] = $id;
			$i++;
		}
		usort($headers,'_cmptitle');
		$i = 0;
		foreach ($headers as $h) {
			$ids[$i] = $h['id'];
			$i++;
		}
	}

	if (config('reversesort') || $sort == 'rsort')
		$ids = array_reverse($ids);
}


/* createrecord - create an empty record */

function createrecord($title = NULL, $body = "") {

	$date = date(config('datestr'));

	if ($title == NULL)
		return "date:$date\ntitle:Title\n\nPlace content here.";

	$record['title'] = $title;
	$record['body'] = $body;
	$record['date'] = $date;
	$record['id'] = 0;

	return $record;
}


/* recorduserdata - add some server data to a record */

function recorduserdata(&$record) {

	if (isset($_SERVER['REMOTE_ADDR']))
		$record['REMOTE_ADDR'] = $_SERVER['REMOTE_ADDR'];
	if (isset($_SERVER['HTTP_USER_AGENT']))
		$record['HTTP_USER_AGENT'] = $_SERVER['HTTP_USER_AGENT'];
	if (isset($_SERVER['HTTP_REFERER']))
		$record['HTTP_REFERER'] = $_SERVER['HTTP_REFERER'];
}


/* _internal functions */


/* _defaultheaders - check for and/or create additional headers */

function _defaultheaders(&$record) {

	$headers = config('headers');
	if (!is_array($headers))
		return;

	foreach ($headers as $key => $val)
		if (!isset($record[$key]))
			$record[$key] = $val;
}


/* _recordrefs - replace record vars (marked by "[]") in a string */

// This substitutes a [word] in body with a record header value, ie.
// with a record header of "title: Foobar", the string "Hey there, [title]!"
// would result in "Hey there, Foobar!". (Of little use but we think it cool.)

function _recordrefs($record,&$body) {

	$re = '/\[([a-z]+)\]/';
	if (preg_match_all($re,$body,$res))
		foreach ($res[0] as $k => $v) {
			$h = $res[1][$k];
			if (isset($record[$h]))
				$body = str_replace($v,$record[$h],$body);
		}
}
